using System;
using System.Linq;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Text;

namespace Ekey.EkeyNet.Sdk
{
	public static class EkeyNetSdk
	{
    public const int MaxArrayLength = 10000;

    public static Answer ServerAnswerFromInt(int n)
		{
			switch (n)
			{
			case -1:
				return Answer.NoData;
			case 0:
				return Answer.Error;
			case 1:
				return Answer.Cmd;
			case 2:
				return Answer.Respone;
			default:
				throw new ArgumentOutOfRangeException("n");
			}
		}

		public static bool SdkSetCommand(Field f, Command cmd)
		{
			IntPtr p = Marshal.AllocHGlobal(sizeof(byte));
			Marshal.WriteByte(p, (byte)cmd);
			int result = EkeyNetSdkWrapper.SetField((Int32)f, p, 0);
			Marshal.FreeHGlobal(p);
			return 0 != result;
		}

		public static bool SdkSetInt32(Field f, Int32 val)
		{
			IntPtr p = Marshal.AllocHGlobal(Marshal.SizeOf(val));
			Marshal.WriteInt32(p, (Int32)val);
			int result = EkeyNetSdkWrapper.SetField((Int32)f, p, 0);
			Marshal.FreeHGlobal(p);
			return 0 != result;
		}

		public static bool SdkSetUInt32(Field f, UInt32 val)
		{
			IntPtr p = Marshal.AllocHGlobal(Marshal.SizeOf(val));
			Marshal.WriteInt32(p, (Int32)val);
			int result = EkeyNetSdkWrapper.SetField((Int32)f, p, 0);
			Marshal.FreeHGlobal(p);
			return 0 != result;
		}

		public static bool SdkSetString(Field f, string val)
		{
			if (string.IsNullOrEmpty(val))
			{
				return false;
			}
			IntPtr p = Marshal.StringToHGlobalUni(val);
			int result = EkeyNetSdkWrapper.SetField((Int32)f, p, 0);
			Marshal.FreeHGlobal(p);
			return 0 != result;
		}

		public static bool SdkGetByteValue(Field f, out byte val)
		{
			val = 0;
			Int32 length = Marshal.SizeOf(val);
			IntPtr p = Marshal.AllocHGlobal(length);
			Marshal.WriteByte(p, 0);
			int result = EkeyNetSdkWrapper.GetField((Int32)f, p, length);
			if (0 != result)
			{
				val = Marshal.ReadByte(p);
			}
			Marshal.FreeHGlobal(p);
			return true;
		}

		public static bool SdkGetCommand(out Command cmd)
		{
			cmd = Command.Logon;
			byte val = 0;
			if (SdkGetByteValue(Field.Cmd, out val))
			{
				cmd = (Command)val;
				return true;
			}
			return false;
		}

		public static bool SdkGetError(out Error e)
		{
			e = Error.Success;
			byte val = 0;
			if (SdkGetByteValue(Field.Answer, out val))
			{
				e = (Error)val;
				return true;
			}
			return false;
		}

		public static bool SdkGetStringValue(Field f, out string val)
		{
			val = null;
			Int32 length = 512 * 2;
			IntPtr p = Marshal.AllocHGlobal(length);
			int result = EkeyNetSdkWrapper.GetField((Int32)f, p, length);
			if (0 != result)
			{
				val = Marshal.PtrToStringUni(p);
			}
			Marshal.FreeHGlobal(p);
			return true;
		}

    public static bool SdkGetStringArrayValue(Field f, out List<string> val)
    {
      val = null;
      Int32 length = MaxArrayLength + 2;
      IntPtr p = Marshal.AllocHGlobal(length);
      int result = EkeyNetSdkWrapper.GetField((Int32)f, p, length);
      if (0 != result)
      {
        byte[] rawString = new byte[result];
        Marshal.Copy(p, rawString, 0, result);

        char[] rawChars = Encoding.Unicode.GetChars(rawString);
        int len = rawChars.Length;

        // Der string muss mit einer doppelten Nullterminierung abgeschlossen sein (C-Style String Array)!
        if (rawChars[len - 1] != '\0' || rawChars[len - 2] != '\0')
        {
          return false;
        }
        Array.Resize(ref rawChars, len - 2);
        string all = new string(rawChars);
        string[] values = all.Split('\0');
        val = values.ToList();
        //val = CharArrayToStrings(Encoding.Unicode.GetChars(rawString));
      }
      Marshal.FreeHGlobal(p);
      return true;
    }

    public static bool SdkGetUIntArrayValue(Field f, out uint[] val)
    {
      val = null;
      Int32 length = MaxArrayLength + 2;
      IntPtr p = Marshal.AllocHGlobal(length);
      int result = EkeyNetSdkWrapper.GetField((Int32)f, p, length);
      if (0 != result)
      {
        if (0 != (result % sizeof(uint)))
        {
          return false;
        }
        uint[] rawArray = new uint[result / sizeof(uint)];
        byte[] rawData = new byte[result];
        Marshal.Copy(p, rawData, 0, result);

        for (int i = 0; i < result / sizeof(uint); ++i)
        {
          rawArray[i] = BitConverter.ToUInt32(rawData, i * sizeof(uint));
        }

        val = rawArray;
      }
      else
      {
        val = Array.Empty<uint>();
      }
      Marshal.FreeHGlobal(p);
      return true;
    }

    /*
    private static List<string> CharArrayToStrings(char[] sequence)
    {
      List<string> strings = new List<string>();
      int startIndex = 0;
      while (true)
      {
        int index;
        try
        {
          index = Array.IndexOf(sequence, '\0', startIndex);
        }
        catch (Exception ex)
        {
          break;
        }
        if (index == -1)
        {
          break;
        }
        if (index + 1 == sequence.Length)
        {
          break;
        }
        int length = index - startIndex;
        string element = new string(sequence, startIndex, length);
        strings.Add(element);
        startIndex += length + 1;
      }
      return strings;
    }
    */
  }
}