﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceProcess;
using System.Windows.Forms;

namespace Ekey.EkeyNet.Sdk.Samples
{
    public partial class MainForm : Form
	{
		private static readonly log4net.ILog logger = log4net.LogManager.GetLogger(typeof(MainForm));

		private bool connected = false;
		private bool loggedOn = false;
		private Timer sdkPoller;
		public MainForm()
		{
			InitializeComponent();
			CenterToScreen();

			for (Command i = Command.ReadUser; i < Command.GetChangeCount; ++i)
			{
				comboBoxCommands.Items.Add(i);
			}
			comboBoxCommands.SelectedIndex = -1;

			listViewAnswer.Columns.Add("Time", 130);
			listViewAnswer.Columns.Add("Response", 315);

			//textBoxUsername.Text = "ekeynetsdk";
			//textBoxPwd.Text = "ekeynetsdk";

			_SetControlsState();
		}

		private void _Destroy()
		{
			if (loggedOn)
			{
				if (0 == EkeyNetSdkWrapper.Logoff())
				{
					MessageBox.Show("Logoff failed.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
				}
				_DestroyPolling();
			}
			loggedOn = false;
			if (connected)
			{
				EkeyNetSdkWrapper.Disconnect();
			}
			connected = false;
			_SetControlsState();
		}

		private void _StartupPolling()
		{
			sdkPoller = new Timer();
			sdkPoller.Interval = 50; //Alle 50 ms pollen
			sdkPoller.Tick += SdkPoller_Tick;
			sdkPoller.Start();
		}

		private void _DestroyPolling()
		{
			if (sdkPoller != null)
			{
				sdkPoller.Stop();
				sdkPoller.Tick -= SdkPoller_Tick;
				sdkPoller = null;
			}
		}

		private void SdkPoller_Tick(object sender, EventArgs e)
		{
			Answer answer = EkeyNetSdk.ServerAnswerFromInt(EkeyNetSdkWrapper.Read());

			if (Answer.NoData == answer)
			{
				// no answer from server try it later
				return;
			}

			if (Answer.Error == answer)
			{
				// maybe ekey net master server is not running (stopped, chrashed, ...)
				_DestroyPolling();
				_Destroy();
				if (logger.IsWarnEnabled)
				{
					logger.Warn("SdkPoller_Tick: Lost connection to the ekey net system. Doing clean communication shutdown.");
				}
				MessageBox.Show("SdkPoller_Tick: Lost connection to the ekey net system. Doing clean communication shutdown.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}

			if (Answer.Respone == answer)
			{
				_HandleServerAnswer();
				return;
			}

			if (Answer.Cmd == answer)
			{
				_HandleServerCmd();
				return;
			}
		}


		private void _HandleServerAnswer()
		{
			Command cmd;
			if (!EkeyNetSdk.SdkGetCommand(out cmd))
			{
				return;
			}
			Error error;
			if (!EkeyNetSdk.SdkGetError(out error))
			{
				return;
			}
			_AddResultLine(string.Format("Command: {0} -> Response: {1}", cmd, error), true);
			List<string> values;
			FieldContentToString(Field.Answer, out values);
			switch (error)
			{
			case Error.Success:
				{
					switch (cmd)
					{
					case Command.AddUser:
					case Command.ReadUser:
						//_AddFieldLine(Field.Company);
						_AddFieldLine(Field.ObjectID);
						_AddFieldLine(Field.Name);
						_AddFieldLine(Field.Firstname);
						_AddFieldLine(Field.Lastname);
						_AddFieldLine(Field.Description);
						return;
          case Command.ReadGroups:
            _AddFieldLine(Field.Usergroups);
            return;
          case Command.ReadGroup:
            _AddFieldLine(Field.ObjectID);
            _AddFieldLine(Field.Usergroup);
            _AddFieldLine(Field.Usergroup2);
            _AddFieldLine(Field.Description);
            _AddFieldLine(Field.UsersInGroup);
            return;
          case Command.ReadTimezones:
            _AddFieldLine(Field.Timezones);
            return;
          }
				}
				break;
			default:
				if (logger.IsErrorEnabled)
				{
					logger.ErrorFormat("_HandleServerAnswer: Command:{0} Result:{1}", cmd, error);
				}
				break;
			}
		}

		private void _HandleServerCmd()
		{
      Command cmd;
      if (!EkeyNetSdk.SdkGetCommand(out cmd))
      {
        return;
      }
      Error error;
      if (!EkeyNetSdk.SdkGetError(out error))
      {
        return;
      }
      _AddResultLine(string.Format("Command: {0} -> Response: {1}", cmd, error), true);
      List<string> values;
      FieldContentToString(Field.Answer, out values);
      switch (error)
      {
      case Error.Success:
        {
          switch (cmd)
          {
          case Command.AddUser:
          case Command.ReadUser:
            //_AddFieldLine(Field.Company);
            _AddFieldLine(Field.ObjectID);
            _AddFieldLine(Field.Name);
            _AddFieldLine(Field.Firstname);
            _AddFieldLine(Field.Lastname);
            _AddFieldLine(Field.Description);
            return;
          case Command.ReadGroups:
            _AddFieldLine(Field.Usergroups);
            return;
          case Command.ReadGroup:
            _AddFieldLine(Field.ObjectID);
            _AddFieldLine(Field.Usergroup);
            _AddFieldLine(Field.Usergroup2);
            _AddFieldLine(Field.Description);
            _AddFieldLine(Field.UsersInGroup);
            return;
          case Command.ReadTimezones:
            _AddFieldLine(Field.Timezones);
            return;
          case Command.NotifyUser:
            _AddFieldLine(Field.UsersInGroup);
            return;
          case Command.NotifyTerminal:
          case Command.NotifySave:        
            return;
          }
        }
        break;
      default:
        if (logger.IsErrorEnabled)
        {
          logger.ErrorFormat("_HandleServerAnswer: Command:{0} Result:{1}", cmd, error);
        }
        break;
      }
    }

		private void _SetControlsState()
		{
			buttonConnect.Enabled = true;
			buttonLogin.Enabled = connected;
			textBoxUsername.Enabled = textBoxPwd.Enabled = connected;
			if (connected)
			{
				buttonConnect.Text = "Disconnect";
				buttonLogin.Enabled = !string.IsNullOrEmpty(textBoxUsername.Text) && !string.IsNullOrEmpty(textBoxPwd.Text);
			}
			else
			{
				buttonConnect.Text = "Connect";
			}

			if (loggedOn)
			{
				buttonLogin.Text = "Logoff";
			}
			else
			{
				buttonLogin.Text = "Login";
			}
			_SetExecControlsState();		
		}

		private void _SetExecControlsState()
		{
			bool enableCmdControls = connected && loggedOn;
			comboBoxCommands.Enabled = enableCmdControls;
			buttonExecute.Enabled = enableCmdControls;
			textBoxDisplayname.Enabled = enableCmdControls;
			textBoxFirstname.Enabled = enableCmdControls;
			textBoxLastname.Enabled = enableCmdControls;
			textBoxDesc.Enabled = enableCmdControls;
			textBoxOid.Enabled = enableCmdControls;
			textBoxPath.Enabled = enableCmdControls;
			textBoxNewName.Enabled = enableCmdControls;
			textBoxPid.Enabled = enableCmdControls;
			textBoxCompany.Enabled = enableCmdControls;

			if (enableCmdControls)
			{
				buttonExecute.Enabled = comboBoxCommands.SelectedIndex != -1;
			}
			else
			{
				buttonExecute.Enabled = false;
			}
		}

		private void ButtonConnect_Click(object sender, EventArgs e)
		{
			if (connected)
			{
				EkeyNetSdkWrapper.Disconnect();
				connected = false;
			}
			else
			{
				ServiceController sc = new ServiceController("ekeynetmasterserver");
				try
				{
					if (ServiceControllerStatus.Running != sc.Status)
					{
						MessageBox.Show("Unable to connect. The service ekey net master server is not running on this computer.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
						return;
					}
				}
				catch (InvalidOperationException ex)
				{
					MessageBox.Show(string.Format("Unable to connect. -> [{0}]", ex.Message), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
					return;
				}	
				if (0 != EkeyNetSdkWrapper.Connect())
				{
					connected = true;
				}
				else
				{
					MessageBox.Show("Establishing connection to the ekey net system with the ekey net sdk failed.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
				}
			}
			_SetControlsState();
		}

		private void MainForm_FormClosed(object sender, FormClosedEventArgs e)
		{
			_Destroy();
		}

		private void ButtonLogin_Click(object sender, EventArgs e)
		{
			if (!connected)
			{
				return;
			}

			if (loggedOn)
			{
				if (0 != EkeyNetSdkWrapper.Logoff())
				{
					loggedOn = false;
					_DestroyPolling();
					//textBoxUsername.Text = textBoxPwd.Text = string.Empty;
				}
			}
			else
			{
				if (0 != EkeyNetSdkWrapper.Logon(textBoxUsername.Text, textBoxPwd.Text))
				{
					loggedOn = true;
					_StartupPolling();
				}
				else
				{
					MessageBox.Show("Login to ekey net failed (check credentials).", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
				}
			}
			_SetControlsState();
		}

		private void TextBoxUsername_TextChanged(object sender, EventArgs e)
		{
			_SetControlsState();
		}

		private void TextBoxPwd_TextChanged(object sender, EventArgs e)
		{
			_SetControlsState();
		}


		private void ComboBoxCommands_SelectedIndexChanged(object sender, EventArgs e)
		{
			_SetExecControlsState();
		}

		private void ButtonExecute_Click(object sender, EventArgs e)
		{
			Command cmd = (Command)comboBoxCommands.SelectedItem;

			string dn, fn, ln, desc, oid, path, newName, pid, company;
			dn = textBoxDisplayname.Text;
			fn = textBoxFirstname.Text;
			ln = textBoxLastname.Text;
			desc = textBoxDesc.Text;
			oid = textBoxOid.Text;
			path = textBoxPath.Text;
			newName = textBoxNewName.Text;
			pid = textBoxPid.Text;
			company = textBoxCompany.Text;

			switch (cmd)
			{
			//Logon 
			//Logoff
			case Command.ReadUser:
				if (!_SetUserLookupFields(oid, dn, fn, ln, pid))
				{
					MessageBox.Show("Invalid search criterias.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
					return;
				}
				break;
			case Command.AddUser:
				if (!string.IsNullOrEmpty(fn))
				{
					EkeyNetSdk.SdkSetString(Field.Firstname, fn);
				}
				if (!string.IsNullOrEmpty(ln))
				{
					EkeyNetSdk.SdkSetString(Field.Lastname, ln);
				}
				if (!string.IsNullOrEmpty(dn))
				{
					EkeyNetSdk.SdkSetString(Field.Name, dn);
				}
				if (!string.IsNullOrEmpty(desc))
				{
					EkeyNetSdk.SdkSetString(Field.Description, desc);
				}
				break;
			case Command.ModifyUser:
				if (!_SetUserLookupFields(oid, dn, fn, ln, pid))
				{
					MessageBox.Show("Invalid search criterias.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
					return;
				}
				if (!string.IsNullOrEmpty(desc))
				{
					EkeyNetSdk.SdkSetString(Field.Description, desc);
				}
				break;
			case Command.DeleteUser:
				if (!_SetUserLookupFields(oid, dn, fn, ln, pid))
				{
					MessageBox.Show("Invalid search criterias.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
					return;
				}
				break;
			case Command.AddUserToGroup:
			case Command.RemoveUserFromGroup:
				if (!_SetUserLookupFields(oid, dn, fn, ln, pid))
				{
					MessageBox.Show("Invalid search criterias.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
					return;
				}
				if (string.IsNullOrEmpty(path))
				{
					MessageBox.Show("Invalid path argument (null or empty).", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
					return;
				}
				EkeyNetSdk.SdkSetString(Field.Usergroup, path);
				break;
			case Command.AddUserToCommonGroup:
				if (!_SetUserLookupFields(oid, dn, fn, ln, pid))
				{
					MessageBox.Show("Invalid search criterias.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
					return;
				}
				break;
			case Command.RemoveUserFromCommonGroup:
				if (!_SetUserLookupFields(oid, dn, fn, ln, pid))
				{
					MessageBox.Show("Invalid search criterias.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
					return;
				}
				break;
			case Command.ReadGroups:
				/*Ist mehr als eine Firma vorhanden kann man durch Angabe von EKEY_NET_FIELD_COMPANY auf eine spezifische Firma zugreifen
					, ansonsten wird die Firma des angemeldeten Benutzers verwendet.*/
				if (!string.IsNullOrEmpty(path))
				{
					EkeyNetSdk.SdkSetString(Field.Usergroup, path);
				}
				break;
			case Command.ReadGroup:
			case Command.AddGroup:
			case Command.DeleteGroup:
				if (string.IsNullOrEmpty(path))
				{
					MessageBox.Show("Invalid path criteria.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
					return;
				}
				EkeyNetSdk.SdkSetString(Field.Usergroup, path);
				break;
			case Command.RenameGroup:
				if (string.IsNullOrEmpty(path) || string.IsNullOrEmpty(newName))
				{
					MessageBox.Show("Invalid parameter for renaming a user group.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
					return;
				}
				EkeyNetSdk.SdkSetString(Field.Usergroup, path);
				EkeyNetSdk.SdkSetString(Field.Usergroup2, newName);
				break;
			case Command.AddTimezone:
				break;
			case Command.ReadTimezone:
				break;
			case Command.ReadTimezones:
				break;
			case Command.EnableRcvNotify:
				break;
			case Command.DisableRcvNotify:
				break;
			case Command.NotifyUser:
				break;
			case Command.NotifyTerminal:
				break;
			case Command.NotifySave:
				break;
			case Command.SendData:
				break;
			case Command.GetChangeCount:
				break;
			}

			Field field = Field.Cmd;
			if (!EkeyNetSdk.SdkSetCommand(field, cmd))
			{
				MessageBox.Show(string.Format("Error setting field -> [{0}: Value:{1}]", field, cmd), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}
			if (0 == EkeyNetSdkWrapper.Write())
			{
				MessageBox.Show(string.Format("Error Writing data -> [{0}: Value:{1}]", field, cmd), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

		private void TextBoxDisplayname_TextChanged(object sender, EventArgs e)
		{

		}

		private void TextBoxFirstname_TextChanged(object sender, EventArgs e)
		{

		}

		private void TextBoxLastname_TextChanged(object sender, EventArgs e)
		{

		}

		private void TextBoxDesc_TextChanged(object sender, EventArgs e)
		{

		}

		private void TextBoxOid_TextChanged(object sender, EventArgs e)
		{

		}

		private void TextBoxPath_TextChanged(object sender, EventArgs e)
		{

		}

		private void TextBoxNewName_TextChanged(object sender, EventArgs e)
		{

		}

		private void TextBoxPid_TextChanged(object sender, EventArgs e)
		{

		}

		private void TextBoxCompany_TextChanged(object sender, EventArgs e)
		{

		}

		private void _AddResultLine(string result, bool addTimeStamp)
		{
			DateTime t = DateTime.Now;
			ListViewItem item = new ListViewItem((addTimeStamp) ? t.ToString() : string.Empty);
			item.SubItems.Add(result);
			listViewAnswer.Items.Add(item);
			listViewAnswer.EnsureVisible(listViewAnswer.Items.Count - 1);
		}

		private void _AddFieldLine(Field f)
		{
			string sField = f.ToString();

			List<string> list;
			FieldContentToString(f, out list);

			int count = 0;
			string str = string.Empty;
      bool isSingleEntry = list.Count == 1;
			foreach (string e in list)
			{
				str = e;
				if (string.IsNullOrEmpty(str))
				{
					continue;
				}
				if (count == 0)
				{
          if (isSingleEntry)
          {
            _AddResultLine(string.Format("{0}: '{1}'", sField, str), false);
          }
          else
          {
            _AddResultLine(string.Format("{0}:", sField), false);
            _AddResultLine(string.Format("        '{0}'", str), false);
          }
				}
				else
				{
					_AddResultLine(string.Format("        '{0}'", str), false);
				}
				++count;
			}
		}

		private bool _SetUserLookupFields(string oid, string name, string fn, string ln, string pid)
		{
			if (string.IsNullOrEmpty(oid) && string.IsNullOrEmpty(name) && string.IsNullOrEmpty(fn) && string.IsNullOrEmpty(ln) && string.IsNullOrEmpty(pid))
			{
				MessageBox.Show("None of the possible user search criterias are valid.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return false;
			}
			if (!string.IsNullOrEmpty(oid))
			{
				Int32 id = 0;
				if (!Int32.TryParse(oid, out id))
				{
					MessageBox.Show(string.Format("The search criteria Object-ID:'{0}' is not a valid number.", oid), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
					return false;
				}
				if (!EkeyNetSdk.SdkSetInt32(Field.ObjectID, id))
				{
					MessageBox.Show(string.Format("Unable to set the Object-ID:'{0}'.", oid), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
					return false;
				}
			}
			if (!string.IsNullOrEmpty(name))
			{
				if (!EkeyNetSdk.SdkSetString(Field.Name, name))
				{
					MessageBox.Show(string.Format("Unable to set the Name:'{0}'.", name), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
					return false;
				}
			}
			if (!string.IsNullOrEmpty(fn) && !string.IsNullOrEmpty(ln))
			{
				if (!EkeyNetSdk.SdkSetString(Field.Firstname, fn))
				{
					MessageBox.Show(string.Format("Unable to set the Firstname:'{0}'.", fn), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
					return false;
				}
				if (!EkeyNetSdk.SdkSetString(Field.Lastname, ln))
				{
					MessageBox.Show(string.Format("Unable to set the Lastname:'{0}'.", ln), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
					return false;
				}
			}
			if (!string.IsNullOrEmpty(pid))
			{
				if (!EkeyNetSdk.SdkSetString(Field.PersonalID, pid))
				{
					MessageBox.Show(string.Format("Unable to set the Personal-ID:'{0}'.", pid), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
					return false;
				}
			}
			return true;
		}

		public static void FieldContentToString(Field f, out List<string> values)
		{
			values = new List<string>();

			switch (f)
			{
			case Field.Cmd:
				{
					Command c;
					if (EkeyNetSdk.SdkGetCommand(out c))
					{
						values.Add(c.ToString());
					}
				}
				break;
			case Field.Answer:
				{
					Error e;
					if (EkeyNetSdk.SdkGetError(out e))
					{
						values.Add(e.ToString());
					}
				}
				break;
			case Field.LogonName:
			case Field.LogonPwd:
			case Field.Company:
			case Field.Name:
			case Field.Firstname:
			case Field.Lastname:
			case Field.Description:
			case Field.PersonalID:
			case Field.Email:
			case Field.Phone:
			case Field.Phone2:
			case Field.Address:
			case Field.Title:
			case Field.Position:
			case Field.Department:
			case Field.Supervisor:
			case Field.Assistant:
			case Field.Userdef1:
			case Field.Userdef2:
			case Field.Userdef3:
			case Field.Userdef4:
			case Field.Userdef5:
			case Field.Userdef6:
			case Field.Userdef7:
			case Field.Userdef8:
			case Field.Userdef9:
			case Field.Userdef10:
			case Field.Usergroup:
			case Field.Usergroup2:
      case Field.Timezone:
				{
					string s;
					if (EkeyNetSdk.SdkGetStringValue(f, out s))
					{
						values.Add(s);
					}
				}
				break;
      case Field.Usergroups:
      case Field.Timezones:
        {
          List<string> entries;
          if (EkeyNetSdk.SdkGetStringArrayValue(f, out entries))
          {
            values.AddRange(entries);
          }
          else
          {
            values.Add(string.Format("Error reading {0}", f.ToString()));
          }
        }
        break;
      case Field.UsersInGroup:
        {
          uint[] members;
          if (EkeyNetSdk.SdkGetUIntArrayValue(f, out members))
          {
            string s = string.Empty;
            if (members.Count() == 0)
            {
              s = "Keine Mitglieder";
            }
            else
            {
              foreach (var i in members)
              {
                if (string.IsNullOrEmpty(s))
                {
                  s += $"{i}";
                }
                else
                {
                  s += $" {i}";
                }
              }
            }

            values.Add(s);
          }
          else
          {
            values.Add(string.Format("Error reading {0}", f.ToString()));
          }
        }
        break;
      }

		}
	}
}
